@@ -1,5 +1,5 @@
 package App::pherkin;
-$App::pherkin::VERSION = '0.54';
+$App::pherkin::VERSION = '0.57';
 use strict;
 use warnings;
 
@@ -19,11 +19,12 @@ use Test::BDD::Cucumber::I18n
 use Test::BDD::Cucumber::Loader;
 
 use Moo;
-use Types::Standard qw( ArrayRef );
+use Types::Standard qw( ArrayRef Bool );
 has 'step_paths' => ( is => 'rw', isa => ArrayRef, default => sub { [] } );
 has 'extensions' => ( is => 'rw', isa => ArrayRef, default => sub { [] } );
 has 'tags'       => ( is => 'rw', isa => ArrayRef, required => 0 );
 has 'tag_scheme' => ( is => 'rw', isa => ArrayRef, required => 0 );
+has 'match_only' => ( is => 'rw', isa => Bool,     default => 0 );
 
 has 'harness' => ( is => 'rw' );
 
@@ -33,7 +34,7 @@ App::pherkin - Run Cucumber tests from the command line
 
 =head1 VERSION
 
-version 0.54
+version 0.57
 
 =head1 SYNOPSIS
 
@@ -78,6 +79,7 @@ sub _pre_run {
     die "No feature files found in $features_path" unless @features;
 
     $executor->add_extensions($_) for @{ $self->extensions };
+    $_->pre_execute($self) for @{ $self->extensions };
 
     Test::BDD::Cucumber::Loader->load_steps( $executor, $_ )
         for @{ $self->step_paths };
@@ -85,10 +87,25 @@ sub _pre_run {
     return ( $executor, @features );
 }
 
+sub _post_run {
+    my $self = shift;
+
+    $_->post_execute() for reverse @{ $self->extensions };
+}
+
+
 sub run {
     my ( $self,     @arguments ) = @_;
     my ( $executor, @features )  = $self->_pre_run(@arguments);
-    return $self->_run_tests( $executor, @features );
+
+    if ( $self->match_only ) {
+        $self->_make_executor_match_only($executor) if $self->match_only;
+        $self->_rename_feature_steps( @features );
+    }
+
+    my $result = $self->_run_tests( $executor, @features );
+    $self->_post_run;
+    return $result;
 }
 
 sub _run_tests {
@@ -103,9 +120,7 @@ sub _run_tests {
             { tags => $self->tag_scheme } );
     }
 
-    $_->pre_execute() for @{ $self->extensions };
     $executor->execute( $_, $harness, $tag_spec ) for @features;
-    $_->post_execute() for reverse @{ $self->extensions };
 
     $harness->shutdown();
     return $harness->result;
@@ -251,6 +266,7 @@ sub _process_arguments {
         tags       => [ 't|tags=s@', [] ],
         i18n       => ['i18n=s'],
         extensions => [ 'e|extension=s@', [] ],
+        match_only => ['m|match'],
     );
 
     GetOptions(
@@ -394,6 +410,9 @@ sub _process_arguments {
     # Store our TagSpecScheme
     $self->tag_scheme( $self->_process_tags( @{ $deref->('tags') } ) );
 
+    # Match only?
+    $self->match_only( $deref->('match_only') );
+
     return ( pop @ARGV );
 }
 
@@ -481,6 +500,44 @@ sub _print_langdef {
     exit;
 }
 
+sub _make_executor_match_only {
+    my ($self, $executor) = @_;
+
+    my $match_sub = sub {
+        my $context = shift;
+        $Test::Builder::Test->ok( 1, "Test matched" );
+        return 1;
+    };
+
+    for my $verb ( keys %{$executor->steps} ) {
+        for my $step_tuple ( @{ $executor->steps->{$verb} } ) {
+            $step_tuple->[1] = $match_sub;
+        }
+    }
+
+    return 1;
+}
+
+sub _rename_feature_steps {
+    my ($self, @features) = @_;
+
+    my %steps;
+    for my $feature ( @features ) {
+        for my $scenario ( $feature->background, @{ $feature->scenarios } ) {
+            next unless $scenario;
+            for my $step ( @{ $scenario->steps } ) {
+                $steps{ $step . '' } = $step;
+            }
+        }
+    }
+
+    for my $step_object ( values %steps ) {
+        $step_object->verb_original(
+            'MATCH MODE: ' . ( $step_object->verb_original || $step_object->verb )
+        );
+    }
+}
+
 =head1 AUTHOR
 
 Peter Sergeant C<pete@clueball.com>
